
function mcmcout=estimate_MS_K_bi_uniformative(Y,K,start)
% by Maximo Camacho and Lola Gadea, 2018
% The authors have used and modify some functions of Sylvia Fruehwirth-Schnatter. 
% of her toolbox for Matlab "Finite Mixture and Markov Switching Models", bayesf Version 2.0
rng('default'); %fixing seed
p1 = genpath('C:\WORKA\New projects\MAXIMO\PAPER1\Revision_JBES\web_rev\general_functions');
path(path,p1) %general auxiliary matlab functions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adjust any of the following to control specification desired %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[N,r]=size(Y);  % number of obs. 
M=2000;       % Number of draws from posterior
burnin=500;  % Number of draws discarded
storeS=M;   % Only the last storeS allocations are stored
mcmcout.entropy=zeros(M,1);     % Stores EN(theta(m)/y)
mcmcout.xi=zeros(M,K,K);      % Stores transition probabilities
mcmcout.mu=zeros(M,r,K);          % Stores means
mcmcout.sigma=zeros(M,r*(r+1)/2,K);       % Stores variances
mcmcout.log_mixlik=zeros(M,1);  % Stores log p(y/theta(m))
mcmcout.log_mixprior=zeros(M,1);  % Stores log p(y/theta(m))?????????
mcmcout.logdet=zeros(M,K);
%smooth=0;         % smooth=0 to skip smoothed probs
%perm=1;           % Permutation sampling   
mcmcout.r=r;
mcmcout.K=K;
mcmcout.M=M;
mcmcout.S=zeros(storeS,N);       % Stores classifications S(m)
mcmcout.clust.prob=zeros(K,N);
mcmcout.clust.Smap=zeros(1,N);
mcmcout.clust.risk=zeros(1,N);
mcmcout.clust.Sident=zeros(1,N);
mcmcout.clust.Ssim=zeros(1,N);
mcmcout.ic.loglikmax=NaN;
mcmcout.ic.aic=NaN;
mcmcout.ic.bic=NaN;
mcmcout.ic.entropy_ml=NaN;
mcmcout.ic.iclbic_ml=NaN;
yp=Y';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%         Independent priors non-hierarchical                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Uninformative priors                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Priors for mu's b0, B0
b0=zeros(r,K);
B0=NaN(r,r,K);  % B0 is the 2x2xK priors of the variances
B0inv=NaN(2,2,K);
C0=NaN(r,r,K);
for k=1:K
   B0k=1000000*eye(r);
   B0(:,:,k)=B0k;
   B0inv(:,:,k)=inv(B0k);
   C0(:,:,k)=eye(r);
end

% Prior variances inv(sigma)~W(c0,C0). We folow Bensmail et al. (1997):
c0=repmat(r,1,K);   % 1xK priors of c. See toolbook page 58      
% See toolbook page 60 / book page 192
logdetC=zeros(1,K);
for k=1:K
   logdetC(k)=log(det(C0(:,:,k)));
end

% Prior of the transition matrix 
% ekk = 2 and ekk'=1/(K-1), if k ne k' pag 366

%  prior of the transition matrix (nonergodic Markov chain, page 316) 
ep=6;et=1/10;
p=kron(ep,eye(K));  
q=kron(et,eye(K)); 
e0=p+[zeros(K,1),q(:,1:end-1)]; % KxK matrix where each row refers to the K parameters of the Dirichlet
etha0=zeros(K,K);      % KxK matrix of transition probs (rows sum to one)
for i=1:K
  gam=gamrnd(e0(i,:),1);
   while i<K && gam(1,i+1)<=0.01
   gam=gamrnd(e0(i,:),1);
   end
   etha0(i,:)=gam./repmat(sum(gam,2),1,K); % 1xK dranws of each row of etha
end

%First classification and initial value of mu1
if start==1 %start with peaks
dis=24; %minimun distance between peaks/trough
elseif start==2 %start with troughs
dis=24; %minimun distance between trough/peak
end
[~,mukmod1,mukmod2,~,Skmod]=Kmodes_bi_empirical(Y,K,start,dis);
mu1=[mukmod1; mukmod2];
S1=Skmod'; %Initial classification

% Defining different WARNINGS to detect unwished situations
% Warning1 counts the percentage of draws in which the algorithm has been unable to generate a bivariate vector of mu that complies with the partition constraints of the reference cycle; 
% Warning2 checks if the estimated mu values are within the range of data values allowing a margin of 1 year at the beginning and end; 
% Warning3 counts the percentage of classifications that leave an empty cluster.
warning1=zeros(M,1);
warning2=0;
warning3=zeros(M,1);
index_bad=zeros(burnin+M,1); % Draws that does not comply the reference cycle restrictions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   Running MCMC algorithm                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Version 2015a Matlab
for m=1:(burnin+M)
  %fprintf('MCMC number %4.0f of %4.0f\n',m,burnin+M);
  mcmcout.ST(m,:)=S1;  % Store classifications
  % 1) Page 330:  Each row trans matrix follows etha0i.~D(e0i1,...e0ik) 
  %    => etha1i.~D(e1i1(S),...,e1iK(S)) where eij(S)=e0ij+Nij(S) and Nij(S)=No{St=j/St-1=i}
  Nij=zeros(K);
  for tr=[S1(1:end-1);S1(2:end)]
    Nij(tr(1),tr(2))=Nij(tr(1),tr(2))+1;
  end
  Nij_nerg=diag(diag(Nij));
  for k=1:K-1
      Nij_nerg(k,k+1)=1;
  end
  e1=Nij_nerg+e0;
  etha1=zeros(K,K);   % KxK matrix of transition probs (rows sum to one)
  for i=1:K
      gam=gamrnd(e1(i,:),1);
      etha1(i,:)=gam./repmat(sum(gam,2),1,K); % 1xK dranws of each row of etha
  end
  etha1=check_etha(etha1); %round errors
  ind=repmat(S1,K,1)==repmat((1:K)',1,N);
  Nk=sum(ind,2)';  % This is 1xK vector Nk(s) 
   
  % 2) Sample variances given means p. 191. If iv2~W[c,C] => iv2/mu,S~W[ck(S),Ck(S)]
  %    where ck(S)=c+0.5*Nk(S) and Ck(S)=C+0.5*sumk(Yk-muk)(Yk-muk)'           
  c1=c0+Nk;
  C1=NaN(r,r,K);
  sigma1=NaN(r,r,K);
  sigma1inv=NaN(r,r,K);
  logdetsigma1=NaN(1,K);
  logdetinv=NaN(1,K);
  for k=1:K
     b=(yp(:,S1==k)-repmat(mu1(:,k),1,Nk(k)));
     C1(:,:,k)=C0(:,:,k)+b*b';
     sigma1(:,:,k)=inv(wishrnd(inv(C1(:,:,k)),c1(k)));   % 2x2 dranws of covariance matrix 
     sigma1inv(:,:,k)=inv(sigma1(:,:,k));
     logdetsigma1(k)=log(det(sigma1(:,:,k)));
     logdetinv(k)=log(det(sigma1inv(:,:,k)));
  end 
  
  % 3) Sample means given variances p. 178. If mu~N[b,B] => mu/v2,S~N[bk(S),Bk(S)]
  %    where inv(Bk(S))=inv(B)+inv(v2)*Nk(S) and bk(S)=Bk(S)*[inv(v2)*Nk(S)*mean(y/S)+inv(B)]*b] 
  B1=NaN(r,r,K);
  B1inv=NaN(r,r,K);
  b1=NaN(r,K);
  for k=1:K
     B1inv(:,:,k)= B0inv(:,:,k)+Nk(k)*sigma1inv(:,:,k); 
     B1(:,:,k)=inv(B1inv(:,:,k));
     b1(:,k)=B1(:,:,k)*(squeeze(B0inv(:,:,k))*b0(:,k)+sigma1inv(:,:,k)*sum(yp(:,S1==k),2));   
  end
   
j=1;
while j<1000
  for k=1:K
      mu1(:,k)=b1(:,k)+chol(B1(:,:,k))'*randn(size(B1(:,:,k),1),1);
      while mu1(1,k)>mu1(2,k) %check peaks before troughts
      mu1(:,k)=b1(:,k)+chol(B1(:,:,k))'*randn(size(B1(:,:,k),1),1);
     end
  end
  result=check_RC(mu1);
  if result==0
      break
  else
      j=j+1;
  end
end
if j==1000
    %disp('With these data and parameters it is not possible to guarantee the identification of the reference cycle')
    index_bad(m)=1;
end


  % 4) Sample classifications given means, variances and weigths Multi-move (p. 342)
  %    by sampling S from (2.5) p[s/theta,y]~p(y/S,theta)*p(S/theta)
  %    4.1) Run the Hamilton filter => compute filtered probabilities
  %               Pr(St/yt,theta) for t=1...K and k=1,...,K
  %      If Y~N[m,V] => f=(2*pi)^(-.5*n)*det(V)^(-.5)*exp[-.5*(Y-m)'inv(V)*(Y-m)]
  llh=zeros(K,N); 
  llh1=-.5*r*(log(2*pi)); 
  for k=1:K
      eps=yp-mu1(:,k*ones(1,N)); 
      llh(k,:)=llh1-.5*logdetsigma1(k)-.5*sum(eps'*squeeze(sigma1inv(:,:,k)).*eps',2)';   
  end
  lh=exp(llh);                   % Likelihood p[y/S,theta(k)]
  p_filt=zeros(K,N+1);           % Kx(N+1) vector of filtered probabilities
  pinit=[1;zeros(K-1,1)];        % Ininial (it colud be ergodic or 1/K)
  p_filt(:,1)=pinit;               
  mixlik=0;
  for t=1:N
      p=(etha1'*p_filt(:,t)).*lh(:,t);  % Numerator of (11.2), Kx1 vector
      st=sum(p,1);                      % Denominator of (11.2) 
      mixlik=mixlik+log(st);            % sum loglike mixture
      p_filt(:,t+1)=p/st;               % filtered (classification) probabilities at t, Kx1 vector Pr(St/yt,theta)
  end 
 
  %    4.2) Sample ST(m) at T from Pr(ST/yT,theta) (page 342: Multi-move sampling)
  S1=ones(N+1,1);  
  S1(N+1,1)=sum(cumsum(p_filt(1:K-1,N+1))<rand)+1;    
  pd=log(p_filt(S1(N+1,1),N+1));    % log(p(S/theta))
  

  %    4.3) For t=T-1,T-2,...1 sample St(m) from Pr(St/St+1,yt,theta)  
  for t=N:-1:1      
    p=p_filt(:,t).*etha1(:,S1(t+1,1)); 
    p=p/sum(p); 
    S1(t,1)=sum(cumsum(p(1:K-1))<rand)+1; 
    pd=pd+log(p(S1(t,1)));  
  end

  S1=S1(2:N+1)';            % 1xN vector of posterior allocations
  p_filt=p_filt(:,2:N+1)';  % NxK matrix of filtered probabilities 

  % 5) Permutation (optional)
  %To identify the model and overcome potential label switching problems, we use the identifiability 
  %constraint that the draws must imply a segmentation of the time span into non-overlapping episodes. 
  %To ensure that the restrictions apply, we employ the rejection sampling, which is achieved by
  %discarding the draws that do not satisfy the constrains and the sampler is implemented again until
  %the condition is satisfied.
  %The permutation sampler described by Fruhwirth-Schnatter (2001) that consists on reordering the
  %values of the draws in order to fulfill the state-identifying restrictions does not apply easily in this multivariate framework

  
  % 6) Pg. 210: Bayesian Maximum A Posteriori (MAP) classification:  mcmcout.clust.Smap is S1 maximizing 
  %    the joint posterior p(theta,S|y)=p(y/theta,S)*p(S/theta)*p(theta)
  
  % Calculate mixprior
  % log(p(theta))=log(p(mu,sigma,theta))=log(p(mu))+log(p(sigma))+log(p(theta))
  
  % log(p(mu)) from log multivariate normal densities mu~N[b0,B0]
  % If Y~N[m,V] => f=(2*pi)^(-.5*n)*det(V)^(-.5)*exp[-.5*(Y-m)'inv(V)*(Y-m)]
  par_logdetinv=zeros(K,1); 
  for k=1:K
     par_logdetinv(k)=log(det(squeeze(B0inv(:,:,k))));  
  end  
  err=repmat(mu1-b0,[1 1 r]);
  errmat=permute(err,[1 3 2]).*permute(err,[3 1 2]);
  fd1=-0.5*sum(r*log(2*pi)+par_logdetinv+squeeze(sum(sum(B0inv.*errmat,1),2)),1);  

  % log p(sigmainv) from log Wishart sigmainv~W[c,C] pag 439 (expressions A.25 and A.26)
  C0b=manipulate_matrix(C0,2); 
  arg_yinvcol=zeros(r*(r+1)/2,K);
  for k=1:K
      arg_yinvcol(:,k)=manipulate_matrix(squeeze(sigma1inv(:,:,k)),1);
  end 
  index=0.5*((1:r).*(2:r+1))';
  trQS=2*sum(arg_yinvcol.*C0b,1)-sum(arg_yinvcol(index,:).*C0b(index,:),1);
  fd2=sum(c0.*logdetC+(c0+(r+1)/2).*logdetinv-trQS-r*(r-1)/4*log(pi),2);  
  fd2=fd2-sum(sum(gammaln(repmat(c0,r,1)+0.5-0.5*repmat((1:r)',1,K)),1),2);
     
  % p(mu,sigma)=p(sigma)*p(mu) => fd=log(p(theta))=log(p(sigma))+log(p(mu))
  fd=fd1+fd2;
  
  % log(etha) from log Gamma pag. 432 (expression A.5)
  fl=gammaln(sum(e0,2))+sum((e0-1).*log(etha1)-gammaln(e0),2);
    
  % logprior=log(theta)=log(p(mu,sigma,etha))=log(p(sigma))+log(p(mu))+log(etha)
  logprior=fd+sum(fl); 
  mixprior=logprior;

  % log-Posterior: log(p(theta/y,S))=log(p(y/theta,S)+log(p(theta))
   mixpost=mixlik+mixprior;  
              
  % mcmcout.clust.Smap is S1 that maximizes log(p(theta,S|y))
  if m>burnin
    % Store log of complete data posterior: log(p(theta,S|y))=log(p(y/theta,S))+log(p(S/theta))+log(p(theta)) 
    %mcmcout.log_cdpost(m-burnin)=mixpost+pd;   
    mcmcout.log_cdpost(m-burnin)=mixlik+pd;  
    if (m-burnin)==1  
        maxcdpost=mcmcout.log_cdpost(1)-1;   
    end
    if mcmcout.log_cdpost(m-burnin)> maxcdpost
        mcmcout.clust.Smap=S1;
    end
  end
 
  
  % 7) Compute entropy (2.7)
  logp=zeros(size(p_filt));
  for k=1:K
      logp(p_filt(:,k)==0,k)=-99;
      logp(p_filt(:,k)~=0,k)=log(p_filt(p_filt(:,k)~=0,k));
  end
  entropy=-sum(sum(logp.*p_filt));

  % Store all the results
  if m>burnin 
     mcmcout.entropy(m-burnin)=entropy;
     mcmcout.log_mixlik(m-burnin)=mixlik;               % Stores log[p(y/theta(m))]
     mcmcout.log_mixprior(m-burnin)=mixprior;           % Stores the log of the prior log[p(theta(m))]
     mcmcout.log_mixpost(m-burnin)=mixpost;
     mcmcout.xi(m-burnin,:,:)=etha1;                    % Stores trans. probs
     mcmcout.mu(m-burnin,:,:)=mu1;                      % Stores means
     mcmcout.sigma(m-burnin,:,:,:)=manipulate_matrix(sigma1,2);  % Stores different elements of VARCOV
     mcmcout.logdet(m-burnin,:)=logdetinv;              % Stores log(det((inv(sigma)))
     mcmcout.weight(m-burnin,:)=marstat(etha1);         % Compute ergodic probabilities from etha1
     
    if  m>(M+burnin-storeS)  
        mcmcout.S(m-(M+burnin-storeS),:)=S1;            % Stores classifications S(m)     
    end
  end
end

index_bad=index_bad(burnin+1:M+burnin);
mcmcout.entropy(index_bad==1,:)=[];
mcmcout.log_mixlik(index_bad==1,:)=[];
mcmcout.log_mixprior(index_bad==1,:)=[];
mcmcout.log_mixpost(:,index_bad==1)=[];
mcmcout.xi(index_bad==1,:,:)=[];
mcmcout.mu(index_bad==1,:,:)=[];
mcmcout.sigma(index_bad==1,:,:)=[];
mcmcout.logdet(index_bad==1,:)=[];
mcmcout.weight(index_bad==1,:)=[];
M2=M-sum(index_bad);

if M2==0
    disp('ERROR: there is not sufficient numbers of draws to achieve identification; probably K is too big for this data')
    return
end


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                       Identification                         %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Permutation sampling/now, we guarantee the identificability removing draws
%that do not hold the restrictions of the reference cycle
%mu_1<mu_2<...<mu_K

 %Clustering
 %mcmcout.clust.Smap %without identification, is computed within the MCMC
 mcmcout.clust.Ssim=compute_similarity_matrix(mcmcout.S,K); %original Sylvia code considers a limit of data N<1000
 
 % identify the mixture model before performing clustering in order to ensure that the
 % labelling is the same
clust.prob=zeros(K,N);


    for k=1:K
        clust.prob(k,:)=sum(mcmcout.S==k,1)/size(mcmcout.S,1) ;
    end

    [~,index]=max(clust.prob,[],2); %re-arrange clust.prob by date
    [~,index2]=sort(index);
    clust.prob=clust.prob(index2,:);
   
    [pmax, Saverage]=max(clust.prob);
    clust.risk=1-pmax;
    clust.Sident=Saverage;  

%[Y,mcmcout.clust.Smap',mcmcout.clust.Ssim',mcmcout.clust.Sident',mcmcout.clust.risk',mcmcout.clust.prob']
mcmcout.clust.prob=clust.prob;
mcmcout.clust.risk=clust.risk;
mcmcout.clust.Sident=clust.Sident;
%[mcmcout.clust.Smap',mcmcout.clust.Sident',mcmcout.clust.Ssim']
mcmcout.clust.Smap=arrange_S(mcmcout.clust.Smap);
mcmcout.clust.Sident=arrange_S(mcmcout.clust.Sident);
mcmcout.clust.Ssim=arrange_S(mcmcout.clust.Ssim);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Information criteria
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mcmcout.log_mixlik= mcmcout.log_mixlik(~isnan(mcmcout.log_mixlik)); %remove NaN elements
mcmcout.entropy= mcmcout.entropy(~isnan(mcmcout.entropy)); %remove NaN elements
[logm, is]=sort(mcmcout.log_mixlik);  % approximate ML estimator
loglikmax=logm(end);
mldraw=is(end);
entropy_ml=mcmcout.entropy(mldraw);
dk=(K-1)+2*K+3*K; %num parameters, (K-1) transition matrix, 2*K means and 3*K covariance matrix
ic.loglikmax=loglikmax;
ic.entropy_ml=entropy_ml;
ic.aic= -2*loglikmax+2*dk;
ic.bic= -2*loglikmax+log(N)*dk;
%ic.bicmc=-2*mean(mcmcout.log.mixlik)+dk*(log(N)-1);
ic.iclbic_ml= ic.bic+2*entropy_ml;
mcmcout.ic=ic;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   Parameter estimation                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if start==1
    P=squeeze(mcmcout.mu(:,1,:));
    T=squeeze(mcmcout.mu(:,2,:)); 
    MU=[mean(P)',mean(T)'];
    fprintf('THE ECONOMY STATRS BY A PEAK\n')
elseif start==2
    P=squeeze(mcmcout.mu(:,2,:));   
    T=squeeze(mcmcout.mu(:,1,:));
    MU=[mean(T)',mean(P)'];
    fprintf('THE ECONOMY STATRS BY A TROUGH\n')
end
[MU, ~]=sort_matrix_ascend(MU,1);
if min(min(MU))<min(min(Y))-1 || max(max(MU))>max(max(Y))+1
    %disp('Results are out of the data range')
end
MU=decimals_to_months_matrix(MU);

if min(min(MU))<min(min(Y))-1 || max(max(MU))>max(max(Y))+1
    %disp('Results are out of the data range')
    warning2=1;
end
warning1=sum(warning1)/(M+burnin);
mcmcout.warning.one=warning1;
mcmcout.warning.two=warning2;

for i=1:M2
    if max(mcmcout.S(i,:))<K
        warning3(i)=1;
    end
end
warning3=sum(warning3)/M2;
mcmcout.warning.three=warning3;

