function MOMENTS=compute_moments(mcmcout)
% compute theoretical moments of a model mixture distribution
% Author: Sylvia Fruehwirth-Schnatter
% Last change: July 18, 2006
%Changed by Lola, 2017

    r=mcmcout.r;
    M=mcmcout.M;
    K=mcmcout.K;
    
    MOMENTS.mean=zeros(M,r);
    MOMENTS.var=zeros(M,r,r); 
    MOMENTS.W=zeros(M,r,r); 
    MOMENTS.B=zeros(M,r,r); 
    MOMENTS.corr=zeros(M,r,r); 
    MOMENTS.Rtr=zeros(M,1); 
    MOMENTS.Rdet=zeros(M,1); 
    MOMENTS.high=zeros(M,r,r*r); 
    MOMENTS.skewness=zeros(M,r);
    MOMENTS.kurtosis=zeros(M,r); 
    MOMENTS.ergodic=zeros(M,1,K); 
    MOMENTS.duration=zeros(M,1,K);
    MOMENTS.eigen=zeros(M,1,K); 
 J=4;
    for m=1:M
        mix.par.mu=squeeze(mcmcout.mu(m,:,:));
        mix.par.sigma=manipulate_matrix(squeeze(mcmcout.sigma(m,:,:)),4);
        mix.weight=mcmcout.weight(m,:);
        mix.xi=squeeze(mcmcout.xi(m,:,:));    
    moments.mean=sum(repmat(mix.weight,r,1).*mix.par.mu,2);    
    moments.var=zeros(r,r);    
    moments.W=zeros(r,r);
    moments.B=zeros(r,r);
    for k=1:K
        moments.var= moments.var+ (mix.par.mu(:,k)*mix.par.mu(:,k)'+squeeze(mix.par.sigma(:,:,k)))*mix.weight(k);
        moments.W= moments.W+ squeeze(mix.par.sigma(:,:,k))*mix.weight(k);
        d=(mix.par.mu(:,k)-moments.mean);
        moments.B= moments.B+ d*d'*mix.weight(k);
    end
    moments.var=moments.var-moments.mean*moments.mean';   
    if r>1
        cd=diag(1./diag(moments.var).^.5);
        moments.corr=cd*moments.var*cd;
    end
    
    moments.Rtr=1-trace(moments.W)/trace(moments.var);    
    moments.Rdet=1-det(moments.W)/det(moments.var);
        moments.high=zeros(r,J); 
    if J>=3 moments.skewness=zeros(r,1); end
    moments.kurtosis=zeros(r,1); 
    for j=1:r
        mixuni=mixturemar(mix,j,K);
        moments.high(j,:)=mixturemomentsnor(mixuni,J,moments.mean(j))'; 
        if J>=3  moments.skewness(j)=moments.high(j,3)/moments.high(j,2)^1.5; end
        if J>=4  moments.kurtosis(j)=moments.high(j,4)/moments.high(j,2)^2; end        
    end    

        moments.ergodic=marstat(mix.xi); 
        moments.duration=1./(1-diag(mix.xi))';
        moments.eigen=sort(eig(mix.xi))';


    MOMENTS.mean(m,:)=moments.mean;
    MOMENTS.var(m,:,:)=moments.var; 
    MOMENTS.W(m,:,:)=moments.W; 
    MOMENTS.B(m,:,:)=moments.B; 
    MOMENTS.corr(m,:,:)=moments.corr; 
    MOMENTS.Rtr(m,:)=moments.Rtr; 
    MOMENTS.Rdet(m,:)=moments.Rdet; 
    MOMENTS.high(m,:,:)=moments.high; 
    MOMENTS.skewness(m,:)=moments.skewness;
    MOMENTS.kurtosis(m,:)=moments.kurtosis; 
    MOMENTS.ergodic(m,:,:)=moments.ergodic; 
    MOMENTS.duration(m,:,:)=moments.duration;
    MOMENTS.eigen(m,:,:)=moments.eigen;     
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mixm=mixturemar(mix,j,K)
% compute marginals of a mixture distribution

% Author: Sylvia Fruehwirth-Schnatter
% Last change: July 15, 2006
  
    if size(j,2)==1    
        % extract a univariate marginal density
        par=struct('mu',mix.par.mu(j,:),'sigma',squeeze(mix.par.sigma(j,j,:))');
        mixm=struct('dist','Normal','K',K,'r',1,'weight',mix.weight,'par',par);      
       end
    
function moments=mixturemomentsnor(mix,J,a) 
zm([1:2:J])=0;
zm([2:2:J])=exp(cumsum(log([1:2:(J-1)])));

moments=zeros(J,1);
for m=2:J
    moments(m)=sum(mix.weight.*(mix.par.mu-a).^m,2);    
    for n=1:m
        cm=(mix.par.mu-a).^(m-n).*(mix.par.sigma).^(n/2)*zm(n); 
        moments(m)=moments(m)+nchoosek(m,n)*sum(mix.weight.*cm,2);
    end
    
end    

    
     